package com.sgbarlow.sourcecontrol;

import gnu.regexp.RESyntax;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.Vector;

import org.gjt.sp.util.Log;

/**
* CVSImplementation of SourceControl interface
*
* @author	Mirco Bova
* @author	based on Stuart Barlow sgbarlow@yahoo.com MSVisualSourceSafeImpl
* @created	November 16, 2004
* @since	Version 0.7
*/
/**
 * @author mbova
 *
 */
public class CVSImpl implements SourceControlInterface {
	//private static RE slashRE = null;
	/**
	 * Perl5 syntax with character classes enabled.
	 * Borrowed from org.gjt.sp.jedit.search.RESearchMatcher for compatability.
	 * @since SourceControl 0.6
	 */
	public static final RESyntax RE_SYNTAX_JEDIT = new RESyntax(
				RESyntax.RE_SYNTAX_PERL5).set(RESyntax.RE_CHAR_CLASSES);

	// data that needs to exist beyond instance

	/**
	* Default cnstr used by the factory.
	*
	*/
	public CVSImpl() throws Exception {
	}

	public void init() throws Exception {
	}

	public Vector historyFile(String file) throws Exception {
		  throw new Exception("CVSImpl - History is not supported!");
	}

	/**
	* CVS 'checkout'
	* 
	*/
	public void getFile(String file, String path, String version) throws Exception {
	  throw new Exception("CVSImpl - GET is not supported!");
	}

	/**
	* CVS 'Edit'
	* @param	file	this is the filename to edit
	* 
	* NOTE: comment is not supported
	*/
	public void checkoutFile(String file, String comment) throws Exception {
		Log.log(Log.DEBUG, CVSImpl.class, "checkout(edit) file: " + file);
		File checkoutFile = new File(file);
		String dir =  checkoutFile.getParent();
		String name = checkoutFile.getName();
		String checkoutCmd = cvsCommand() + checkoutCommand() + "\"" + name + "\"";
		execProcess(checkoutCmd,dir);
	}

	/**
	* CVS 'commit'
	*
	*/
	public void checkinFile(String file, String comment) throws Exception {
		String cmdline = cvsCommand() + checkinCommand() + messageBit(comment) + file;
		execProcess(cmdline);
	}

	public void addFile(String file, String filePath, String comment) throws Exception {
		String addCmd = cvsCommand() + addCommand() + "\"" + file + "\"";
		execProcess(addCmd);
	}

	// CVS unedit
	public void undoCheckoutFile(String file) throws Exception {
		Log.log(Log.DEBUG, CVSImpl.class, "undo-checkout(edit) file: " + file);
		File checkoutFile = new File(file);
		String dir =  checkoutFile.getParent();
		String name = checkoutFile.getName();
		String cmdline = cvsCommand() + undoCheckoutCommand() + checkoutFile;
		execProcess(cmdline,dir);

	}

	public String diffFile(String file) throws Exception {
		String diffCmd = cvsCommand() + diffCommand() + file;
		return execProcess(diffCmd);
	}

	private String checkoutCommand() {
		return "edit ";
	}

	private String checkinCommand() {
		return "ci ";
	}

	private String addCommand() {
		return "add ";
	}
	
	private String undoCheckoutCommand() {
		return "unedit ";
	}

	private String historyCommand() {
		return "history ";
	}

	private String diffCommand() {
		return "diff ";
	}
	// in CVS checkout, co and get are synonims
	private String getCommand() {
		return "co ";
	}

	private String exportCommand() {
		return "export ";
	}

	private String tagBit(String tag) {
		if (tag == null)
			return " ";

		return " -r " + tag;
	}

	private String redirectBit(File f) {
		return " -d " + f.getPath();
	}

	private String redirectBit(String fileName) {
		return " -d " + fileName;
	}

	private String messageBit(String comment) {
		return " -m \"" + comment + "\"";
	}

	private String cvsUser() {
		return "";
	}

	/**
	 * CVS executable MUST be in the $PATH
	 * @return
	 */
	private String cvsCommand() {
		return "cvs ";
	}
	
	private static String execProcess(String command) throws IOException {
		String retVal = "";
	
		// assumes process will run in a working dir i.e.
		// cvs will get repository info from a CVS subdir
		Log.log(Log.DEBUG, CVSImpl.class, "execProcess: " + command);

		Process proc = Runtime.getRuntime().exec(command);

		InputStream is = proc.getInputStream();
		InputStream es = proc.getErrorStream();


		try {
			int i = 1;
			String s;

			StringBuffer out = new StringBuffer(1024);
			BufferedReader outReader = new BufferedReader(new InputStreamReader(is));

			Log.log(Log.DEBUG, CVSImpl.class, "-- STDOUT");

			while ((s = outReader.readLine()) != null) {
				if (s.length() == 0)
					continue;

				Log.log(Log.DEBUG, CVSImpl.class, i + ":" + s);

				out.append(s);
				out.append(new String(System.getProperty("line.separator")));
				i++;
			}

			if (out.length() > 0) {
				retVal = out.toString();
			} else {
				Log.log(Log.DEBUG, CVSImpl.class, "nothing");
			}

			StringBuffer err = new StringBuffer(1024);
			BufferedReader errReader = new BufferedReader(new InputStreamReader(es));

			Log.log(Log.DEBUG, CVSImpl.class, "-- STDERR");

			while ((s = errReader.readLine()) != null) {
				if (s.length() == 0)
					continue;

				Log.log(Log.DEBUG, CVSImpl.class, i + ":" + s);

				err.append(s);
				err.append(new String(System.getProperty("line.separator")));
				i++;
			}

			if (err.length() > 0) {
				Log.log(Log.DEBUG, CVSImpl.class, "STDERR - " + err.toString());
				throw new IOException(err.toString());
			}

			proc.waitFor();
		} catch (Exception ex) {
			throw new IOException(ex.getMessage());
		}

		return retVal;
	}
	
	private static String execProcess(String command, String dirname) throws IOException {
		String retVal = "";
		File dir = new File(dirname);
		
		// assumes process will run in a working dir i.e.
		// cvs will get repository info from a CVS subdir
		Log.log(Log.DEBUG, CVSImpl.class, "execProcess: " + command + " in  folder " + dir);

		Process proc = Runtime.getRuntime().exec(command, null, dir);

		InputStream is = proc.getInputStream();
		InputStream es = proc.getErrorStream();


		try {
			int i = 1;
			String s;

			StringBuffer out = new StringBuffer(1024);
			BufferedReader outReader = new BufferedReader(new InputStreamReader(is));

			Log.log(Log.DEBUG, CVSImpl.class, "-- STDOUT");

			while ((s = outReader.readLine()) != null) {
				if (s.length() == 0)
					continue;

				Log.log(Log.DEBUG, CVSImpl.class, i + ":" + s);

				out.append(s);
				out.append(new String(System.getProperty("line.separator")));
				i++;
			}

			if (out.length() > 0) {
				retVal = out.toString();
			} else {
				Log.log(Log.DEBUG, CVSImpl.class, "nothing");
			}

			StringBuffer err = new StringBuffer(1024);
			BufferedReader errReader = new BufferedReader(new InputStreamReader(es));

			Log.log(Log.DEBUG, CVSImpl.class, "-- STDERR");

			while ((s = errReader.readLine()) != null) {
				if (s.length() == 0)
					continue;

				Log.log(Log.DEBUG, CVSImpl.class, i + ":" + s);

				err.append(s);
				err.append(new String(System.getProperty("line.separator")));
				i++;
			}

			if (err.length() > 0) {
				Log.log(Log.DEBUG, CVSImpl.class, "STDERR - " + err.toString());
				throw new IOException(err.toString());
			}

			proc.waitFor();
		} catch (Exception ex) {
			throw new IOException(ex.getMessage());
		}

		return retVal;
	}

}

