package com.sgbarlow.sourcecontrol;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.net.URL;
import java.util.Vector;

import javax.swing.ImageIcon;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTextArea;
import javax.swing.JTree;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;

import org.gjt.sp.jedit.View;
import org.gjt.sp.jedit.jEdit;


public class FileHistoryGraphPanel extends JDialog {
	public static int WIDTH = 400;
	public static int HEIGHT = 400;
	public static String TITLE = "File History Tree - ";

	JTextArea textArea = new JTextArea("Select a Version/Label to view detalis.");

	JTree tree;
	View view;

	//DefaultTreeCellRenderer renderer;
	TreeRenderer renderer = new TreeRenderer();

	String fileName;
	Vector fileVersions;
	DefaultMutableTreeNode fileNode;

	public FileHistoryGraphPanel(View view, String fileName, Vector versions, String position) {

		super(view, "History of " + fileName, false);
		this.view = view;
		setDefaultCloseOperation(FileHistoryGraphPanel.DISPOSE_ON_CLOSE);

		this.fileName = fileName;
		this.fileVersions = versions;

		setupTree();
		layoutComponents();

		//setupEventHandlers();
		setSize(WIDTH, HEIGHT);
		this.pack();
		this.setLocationRelativeTo(view);
		this.setVisible(true);
	}

	void setupTree() {
		fileNode = new DefaultMutableTreeNode(fileName);

		for (int i = 0; i < fileVersions.size(); i++) {

				FileVersion version = (FileVersion)fileVersions.get(i);
			DefaultMutableTreeNode versionNode = new DefaultMutableTreeNode(version);

				for (int j = 0; j < version.labels.size(); j++) {
				FileLabel label = (FileLabel)version.labels.get(j);
					 DefaultMutableTreeNode childNode = new DefaultMutableTreeNode(label);
				versionNode.add(childNode);
			}
			fileNode.add(versionNode);
		}
		tree = new JTree(fileNode);

		//"Angled", "Horizontal", and "None" (the default).
		tree.putClientProperty("JTree.lineStyle", "Horizontal");
		tree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
		tree.setCellRenderer(renderer);
		tree.addTreeSelectionListener(new TreeHandler());
	}

	public void layoutComponents() {

		//Create the scroll pane and add the tree to it.
		JScrollPane treeView = new JScrollPane(tree);

		//Setup a text area.
		textArea.setLineWrap(true);
		textArea.setWrapStyleWord(true);
		textArea.setEditable(false);
		textArea.setFont(jEdit.getFontProperty("metal.secondary.font"));

		//Create the info viewing pane.
		JScrollPane infoView = new JScrollPane(textArea);

		infoView.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		infoView.setPreferredSize(new Dimension(250, 250));

		//Add the scroll panes to a split pane.
		// JSplitPane splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
		JSplitPane splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
		splitPane.setTopComponent(treeView);
		splitPane.setBottomComponent(infoView);

		infoView.setMinimumSize(new Dimension(100, 25));
		treeView.setMinimumSize(new Dimension(200, 25));

		splitPane.setDividerLocation(200);	//XXX: ignored in some releases of Swing. bug 4101306

		  //workaround for bug 4101306:
		  //treeView.setPreferredSize(new Dimension(100, 100));

		  splitPane.setPreferredSize(new Dimension(500, 300));

		  //Add the split pane to this frame.
		  getContentPane().add(splitPane, BorderLayout.CENTER);
	}

	// for debug only
	public static void test(View view, String position) {

		Vector hs = new Vector();

		FileVersion fv2 = new FileVersion();
		fv2.version = "2";
		fv2.user = "gianni";
		fv2.date = "26/03/03";
		fv2.time = "12.30";
		fv2.comment = "Seconda versione";

		FileLabel fla = new FileLabel();
		fla.label = "V_001_001";
		fla.user = "antonio";
		fla.date = "26/03/03";
		fla.time = "12.30";
		fla.comment = "Seconda versione";
		fv2.labels.add(fla);
		hs.add(fv2);

		FileVersion fv1 = new FileVersion();
		fv1.version = "1";
		fv1.user = "mirco";
		fv1.date = "26/02/03";
		fv1.time = "1.22";
		fv1.comment = "Prima versione";
		hs.add(fv1);

		FileHistoryGraphPanel f = new FileHistoryGraphPanel(view, "foo", hs, "");
	}

	public class WindowHandler extends WindowAdapter {
		public void windowClosing(WindowEvent e) {
			System.exit(0);
		}
	}

	public class TreeHandler implements TreeSelectionListener {
		public void valueChanged(TreeSelectionEvent e) {
			String crlf = System.getProperty("line.separator");

			TreePath path = e.getPath();
			String text = path.getPathComponent(path.getPathCount() - 1).toString();
			DefaultMutableTreeNode node = (DefaultMutableTreeNode)
													tree.getLastSelectedPathComponent();

			Object nodeInfo = node.getUserObject();

			if (nodeInfo instanceof FileLabel) {
				FileLabel label = (FileLabel)nodeInfo;
				text = label.label + ": " + label.date + " " + label.time + " - " + label.user;
			} 
			//Build the text for the FileVersion to be displayed to the user
			else if (nodeInfo instanceof FileVersion) {
				FileVersion version = (FileVersion)nodeInfo;
				text = "Version: " + version.version;
				text += crlf + "Date: " + version.date + version.time;
				text += crlf + "User: " + version.user;
				if (node.getChildCount() > 0) {
					text += crlf + "Label(s): ";
					for (int i = node.getChildCount() - 1;;) {
						DefaultMutableTreeNode labelNode = (DefaultMutableTreeNode) node.getChildAt(i);

						FileLabel label = (FileLabel) labelNode.getUserObject();
	
						text += label.label;

						if (--i < 0)
							break;
						text = text + ", ";
					}
				}
				text += ".";
			}

			textArea.setText(text);
		}
	}

	class TreeRenderer extends DefaultTreeCellRenderer {
		private JLabel treeCellRenderer = null;

		public Component getTreeCellRendererComponent(JTree tree, Object value,
				boolean sel, boolean expanded,
				boolean leaf, int row,
				boolean focus) {
			super.getTreeCellRendererComponent(tree, value, sel, expanded, leaf, row, focus);

			if (treeCellRenderer == null) {
				treeCellRenderer =
					new JLabel();
			}

			//treeCellRenderer.setText(value.toString() + "                                ");
			treeCellRenderer.setText(value.toString());

			DefaultMutableTreeNode node = (DefaultMutableTreeNode)value;
			Object nodeInfo = (node.getUserObject());

			Font leafFont=treeCellRenderer.getFont();
			leafFont=leafFont.deriveFont(Font.PLAIN);
			treeCellRenderer.setFont(leafFont);

			if (nodeInfo instanceof FileLabel) {
				URL url = getClass().getResource("/icons/bookmark-item.gif");
				treeCellRenderer.setIcon(new ImageIcon(url));
			} else if (nodeInfo instanceof FileVersion) {
				URL url;
				if (expanded)
					url = getClass().getResource("/icons/dir-open.gif");
				else
					url = getClass().getResource("/icons/dir-closed.gif");
				treeCellRenderer.setIcon(new ImageIcon(url));
				treeCellRenderer.setText("Version " + treeCellRenderer.getText());

				leafFont=leafFont.deriveFont(Font.BOLD);
				treeCellRenderer.setFont(leafFont);
			} else {
				URL url = getClass().getResource("/icons/file-icon.gif");
				treeCellRenderer.setIcon(new ImageIcon(url));
			}

			return treeCellRenderer;
		}
	}

}
