package com.sgbarlow.sourcecontrol;

import java.util.*;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.*;
import java.lang.Process;

import gnu.regexp.*;

import org.gjt.sp.util.Log;

// jedit imports
import org.gjt.sp.jedit.*;

/**
* MS VSSImplementation of SourceControl interface
*
* Version 0.7 contributed by Craig Boal, adapted by Mirco Bova
*
* @author Stuart Barlow
*/
public class MSVisualSourceSafeImpl implements SourceControlInterface {
	//private static RE slashRE = null;
	/**
	 * Perl5 syntax with character classes enabled.
	 * Borrowed from org.gjt.sp.jedit.search.RESearchMatcher for compatability.
	 * @since SourceControl 0.6
	 */
	public static final RESyntax RE_SYNTAX_JEDIT = new RESyntax(
				RESyntax.RE_SYNTAX_PERL5).set(RESyntax.RE_CHAR_CLASSES);

	// data needs to exist beyond instance
	private static String m_ssDir = null;
	private static String m_shareDir = null;
	private static String m_username = null;
	private static String m_password = null;
	private static List m_projects = null;

	private static boolean s_isInitialised = false;

	/**
	* Default cnstr used by the factory.
	*
	*/
	public MSVisualSourceSafeImpl() throws Exception {}

	public void init() throws Exception {
		if ( !s_isInitialised ) {
			Log.log(Log.DEBUG, MSVisualSourceSafeImpl.class, "start init");
			String vssPath = jEdit.getProperty(SourceControlPlugin.OPTION_PREFIX + "filepath");
			String sharePath = jEdit.getProperty(SourceControlPlugin.OPTION_PREFIX + "sharepath");
			List vssProjects = VSSProject.factory(SourceControlPlugin.getGlobalProperties());
			setVSSProjects(vssProjects);
			setSSDir(vssPath);
			setShareDir(sharePath);

			String username = jEdit.getProperty(SourceControlPlugin.OPTION_PREFIX + "vssusername");
			String password = jEdit.getProperty(SourceControlPlugin.OPTION_PREFIX + "vsspassword");
			setUsrAndPwd(username, password);

			Log.log(Log.DEBUG, MSVisualSourceSafeImpl.class, "finish init");

			s_isInitialised = true;
		}
	}

	public static void setUsrAndPwd(String usr, String pwd) {
		if (usr == null)
			return;

		if (usr.length() > 0) {
			m_username = usr;

			if (pwd == null)
				return;

			if (pwd.length() > 0) {
				m_password = pwd;
			}
		}
	}

	public static void setSSDir(String directory) throws Exception {
		File sourceSafe = new File(directory);
		if ( ! sourceSafe.isDirectory() ) {
			throw new Exception("MSVisualSourceSafeImpl - specified VSS directory does not exist - " + directory);
		}
		m_ssDir = directory;
	}

	public static void setShareDir(String directory) throws Exception {
		File sourceSafe = new File(directory);
		if ( ! sourceSafe.isDirectory() ) {
			throw new Exception("MSVisualSourceSafeImpl - specified Share directory does not exist - " + directory);
		}
		m_shareDir = directory;
	}

	public static void setVSSProjects(List projects) throws Exception {
		m_projects = projects;
	}

	private static final int IDLE = 0;

	private static final int GET_LBL_LABEL = 1;
	private static final int GET_LBL_USR_DATE_TIME = 2;
	private static final int GET_LBL_ACTION = 3;
	private static final int GET_LBL_COMMENT = 4;

	private static final int GET_VER_USR_DATE_TIME = 12;
	private static final int GET_VER_ACTION = 13;
	private static final int GET_VER_COMMENT = 14;

	public Vector historyFile(String file) throws Exception {

		Vector hs = new Vector();
		Vector labels = new Vector();
		FileVersion fv;
		FileLabel fl = null;
		fv = null;
		BufferedReader in;
		String line;
		Log.log(Log.DEBUG, MSVisualSourceSafeImpl.class, "history file: " + file);

		String ssFile = convertFileNameToVSSFileName(file);
		Log.log(Log.DEBUG, MSVisualSourceSafeImpl.class, "vss file: " + ssFile);
		String historyCmd = ssCommand() + historyCommand() + ssFile + postStdBit();
		try {
			// let's start and parse the history output
			// the first line should contain the file name
			in = new BufferedReader(new StringReader(new String(execProcess(historyCmd))));
			line = in.readLine();
			if (line.startsWith("History")) {
				// this is the first line and contains the filename
				// and full ss file path
			}
			RE testRE;
			REMatch rem;
			//Loop through all of the lines of the version history
			while ((line = in.readLine()) != null) {
				if (line.length() == 0)
					continue;

				//Set the version

				if (line.startsWith("*****************  Version ")) {
					//If the line is Version and fv is set then we have a completed file version object to add to the history vector
					if (fv != null){
						Log.log(Log.DEBUG, MSVisualSourceSafeImpl.class, "Adding Version: " + fv.version);
						hs.add(fv);
					}
					//Create the new FileVersion object and set it version number
					testRE = new RE("Version (\\d+)", 0, RE_SYNTAX_JEDIT);
					rem = testRE.getMatch(line);
					if (rem != null) {
						//step = GET_VER_USR_DATE_TIME;
						fv = new FileVersion();
						fv.version = new String(rem.toString(1));
						fv.labels = labels;
						labels = new Vector(); //inizializzo le label
						fl = null;
					}
				}
				//Set the label
				else if (line.startsWith("Label:")) {
					if (fl != null) {
						Log.log(Log.DEBUG, MSVisualSourceSafeImpl.class, "Adding lABEL: " + fl.label);
						labels.add(fl);
					}
				
					testRE = new RE("Label: \\\"(.+)\\\"", 0, RE_SYNTAX_JEDIT);
					rem = testRE.getMatch(line);
					if (rem != null) {
						fl = new FileLabel();
						fl.label = new String(rem.toString(1));
					}
				}
				//Set the label comment
				else if (line.startsWith("Label comment:")) {
					testRE = new RE("Label comment: (.+)", 0, RE_SYNTAX_JEDIT);
					rem = testRE.getMatch(line);
					if (rem != null) {
						fl.comment = new String(rem.toString(1));
					}
				}
				//Set the user date and time
				else if (line.startsWith("User")){
					testRE = new RE("User: (.+)Date: (.+)Time: (.+)", 0, RE_SYNTAX_JEDIT);
					rem = testRE.getMatch(line);
					if (rem != null) {
						fv.user = new String(rem.toString(1));
						fv.date = new String(rem.toString(2));
						fv.time = new String(rem.toString(3));
					}
				}
				//Set the comment
				else if (line.startsWith("Comment")){
					testRE = new RE("Comment: (.+)", 0, RE_SYNTAX_JEDIT);
					rem = testRE.getMatch(line);
					if (rem != null) {
						fv.comment = new String(rem.toString(1));
					}
				}
				//Set the action
				else if (line.length() != 0) {
					fv.action = line;
				}
			}
			//Add the last parsed file version
			if (fv != null){
				Log.log(Log.DEBUG, MSVisualSourceSafeImpl.class, "Adding Version: " + fv.version);
				hs.add(fv);
			}
		} catch (Exception ex) {
			Log.log(Log.ERROR, MSVisualSourceSafeImpl.class, ex);
		}
		return hs;
	}

	public void getFile(String file, String path, String version) throws Exception {
		String filePathBit;

		Log.log(Log.DEBUG, MSVisualSourceSafeImpl.class, "get file: " + file);

		File checkoutFile = new File(file);

		if(path == null) {
			filePathBit = ssFileLocation(checkoutFile);
		} else {
			filePathBit = ssFileLocationNotParent(new File(path));
		}

		String ssFile = convertFileNameToVSSFileName(file);
		Log.log(Log.DEBUG, MSVisualSourceSafeImpl.class, "vss file: " + ssFile);

		String checkoutCmd = ssCommand() + getCommand() + ssFile + filePathBit + versionBit(version) + postStdBit();
		execProcess(checkoutCmd);
	}

	/**
	*
	* @param file this is the filename to checkout from VSS
	*/
	public void checkoutFile(String file, String comment) throws Exception {
		Log.log(Log.DEBUG, MSVisualSourceSafeImpl.class, "checkout file: " + file);
		File checkoutFile = new File(file);
		String filePathBit = ssFileLocation(checkoutFile);

		String ssFile = convertFileNameToVSSFileName(file);
		Log.log(Log.DEBUG, MSVisualSourceSafeImpl.class, "vss file: " + ssFile);
		String checkoutCmd = ssCommand() + checkoutCommand() + ssFile + filePathBit + postStdBit(comment);
		execProcess(checkoutCmd);
	}

	/**
	*
	*
	*/
	public void checkinFile(String file, String comment) throws Exception {
		File checkinFile = new File(file);
		String filePathBit = ssFileLocation(checkinFile);
		String ssFile = convertFileNameToVSSFileName(file);

		String checkoutCmd = ssCommand() + checkinCommand() + ssFile + filePathBit + postStdBit(comment);
		execProcess(checkoutCmd);
	}

	/**
	*
	*
	*/
	public void addFile(String file, String filePath, String comment) throws Exception {
		File addFile = new File(file);
		String filePathBit = ssFileLocation(addFile);
		//Set the current project based on the path of the selected file
		setActiveProject(filePath);
		String addCmd = ssCommand() + addCommand() + "\"" + file + "\"" + postStdBit(comment);
		execProcess(addCmd);
	}
	
	/**
	*
	*
	*/
	public void undoCheckoutFile(String file) throws Exception {
		File checkinFile = new File(file);
		String filePathBit = ssFileLocation(checkinFile);
		String ssFile = convertFileNameToVSSFileName(file);

		String checkoutCmd = ssCommand() + undoCheckoutCommand() + ssFile + filePathBit + postStdBit();
		execProcess(checkoutCmd);
	}

	/**
	*
	*
	*/
	public String diffFile(String file) throws Exception {
		File checkinFile = new File(file);
		String ssFile = convertFileNameToVSSFileName(file);
		String localFile = ssFileLocationNotParentNoGL(checkinFile);

		String diffCmd = ssCommand() + diffCommand() + ssFile + localFile + postStdBit();
		return execProcess(diffCmd);
	}

	private String checkoutCommand() {
		return "Checkout ";
	}

	private String checkinCommand() {
		return "Checkin ";
	}

	private String addCommand() {
		return "Add ";
	}
	
	private String undoCheckoutCommand() {
		return "Undocheckout ";
	}

	private String historyCommand() {
		return "History ";
	}

	private String diffCommand() {
		return "Diff ";
	}

	private String getCommand() {
		return "Get ";
	}

	private String versionBit(String version) {
		if (version == null)
			return " ";

		return " -V" + version;
	}

	private String redirectBit(File f) {
		return " -O@" + f.getPath();
	}

	private String redirectBit(String fileName) {
		return " -O@" + fileName;
	}

	private String postStdBit(String comment) {
		return postStdBit() + " \"-C" + comment + "\"";
	}

	private String postStdBit() {
		return ssUser() + " -I-Y";
	}

	private String ssUser() {
		String optionUsrAndPwd = new String("");

		// it seems that -Y alone has no effetcs thus there is no need to check
		// if a username and password have been specified.

		// I was wrong! if username and pwd were not specified
		// an error occurs!!!

		if(m_username == null) {
			return optionUsrAndPwd;
		}

		optionUsrAndPwd = " -Y" + m_username;

		if(m_username != null) {
			optionUsrAndPwd = optionUsrAndPwd + "," + m_password;
		}

		return optionUsrAndPwd;
	}

	private String ssFileLocation(File theFile) {
		return " -GL\"" + theFile.getParentFile().getPath() + "\"";
	}

	private String ssFileLocationNotParent(File theFile) {
		return " -GL\"" + theFile.getPath() + "\"";
	}

	private String ssFileLocationNotParentNoGL(File theFile) {
		// now handles filenames with spaces
		return " \"" + theFile.getPath() + "\"";
	}

	private String ssCommand() {
		String sep = File.separator;
		return m_ssDir + sep + "ss ";
	}

	private String convertFileNameToVSSFileName(String fileName) throws Exception {
		StringBuffer strBuf = new StringBuffer(fileName.toLowerCase());

		boolean found = false;
		VSSProject proj = null;
		Iterator it = m_projects.iterator();
		Log.log(Log.DEBUG, MSVisualSourceSafeImpl.class, "Number of vss projects: " + m_projects.size());
		while (it.hasNext()) {
			proj = (VSSProject)it.next();
			Log.log(Log.DEBUG, MSVisualSourceSafeImpl.class, proj.toString());
			// Mirco -- BEG
			// If we are running on win platforms the check shouldn't be case sesitive
			if ((fileName.toLowerCase()).indexOf((proj.getWorkingFolder()).toLowerCase()) != -1) {
				found = true;
				break;
			}
		}

		if ( !found ) {
			throw new Exception("MSVisualSourceSafeImpl - file is not in working folder of specified VSS Projects - set projects in the VSSIntegration options");
		} else {
			strBuf = strBuf.replace(0, proj.getWorkingFolder().length(), proj.getProjName());
		}

        String ssFileName = strBuf.toString();

        // vss uses forward slash
        ssFileName = ssFileName.replace('\\', '/');
        
        // add quotes around name
        StringBuffer strBufQuotes = new StringBuffer();
        strBufQuotes.append("\"");
        strBufQuotes.append(ssFileName);
        strBufQuotes.append("\"");

		return strBufQuotes.toString();
	}

	/**
	*this function sets the active project for the selected file based on the directory it is in
	*
	*/
	private void setActiveProject(String filePath) throws Exception {
		StringBuffer strBuf = new StringBuffer(filePath.toLowerCase());

		boolean found = false;
		VSSProject proj = null;
		Iterator it = m_projects.iterator();
		Log.log(Log.DEBUG, MSVisualSourceSafeImpl.class, "Number of vss projects: " + m_projects.size());
		while (it.hasNext()) {
			proj = (VSSProject)it.next();
			Log.log(Log.DEBUG, MSVisualSourceSafeImpl.class, proj.toString());
			Log.log(Log.DEBUG, MSVisualSourceSafeImpl.class, "file path: " + filePath.toLowerCase());
			Log.log(Log.DEBUG, MSVisualSourceSafeImpl.class, "project active directory: " + proj.getWorkingFolder().toLowerCase());
			if (filePath.toLowerCase().equals(proj.getWorkingFolder().toLowerCase())) {
				Log.log(Log.DEBUG, MSVisualSourceSafeImpl.class, "Project found");
				found = true;
				break;
			}
		}

		if ( !found ) {
			throw new Exception("MSVisualSourceSafeImpl - file is not in working folder of specified VSS Projects - set projects in the VSSIntegration options");
		} else {
			Log.log(Log.DEBUG, MSVisualSourceSafeImpl.class, "Set active project = " + proj.getProjName());
			execProcess(ssCommand() + "CP \"" + proj.getProjName() + "\"" + postStdBit());
		}
	}
	
	private static String execProcess(String command) throws IOException {
		String retVal = "";
		String[] envp = new String[1];
		String usernameAndPassword = null;


		// share path should be set in the envorinoment
		envp[0] = "SSDIR=" + m_shareDir;
		Log.log(Log.DEBUG, MSVisualSourceSafeImpl.class, "execProcess: " + command + "| envp:" + envp[0]);

		Process proc = Runtime.getRuntime().exec(command, envp);

		InputStream is = proc.getInputStream();
		InputStream es = proc.getErrorStream();


		try {
			int i = 1;
			String s;

			StringBuffer out = new StringBuffer(1024);
			BufferedReader outReader = new BufferedReader(new InputStreamReader(is));

			Log.log(Log.DEBUG, MSVisualSourceSafeImpl.class, "-- STDOUT");

			while ((s = outReader.readLine()) != null) {
				if (s.length() == 0)
					continue;

				Log.log(Log.DEBUG, MSVisualSourceSafeImpl.class, i + ":" + s);

				out.append(s);
				out.append(new String(System.getProperty("line.separator")));
				i++;
			}

			if (out.length() > 0) {
				retVal = out.toString();
			} else {
				Log.log(Log.DEBUG, MSVisualSourceSafeImpl.class, "nothing");
			}

			StringBuffer err = new StringBuffer(1024);
			BufferedReader errReader = new BufferedReader(new InputStreamReader(es));

			Log.log(Log.DEBUG, MSVisualSourceSafeImpl.class, "-- STDERR");

			while ((s = errReader.readLine()) != null) {
				if (s.length() == 0)
					continue;

				Log.log(Log.DEBUG, MSVisualSourceSafeImpl.class, i + ":" + s);

				err.append(s);
				err.append(new String(System.getProperty("line.separator")));
				i++;
			}

			if (err.length() > 0) {
				Log.log(Log.DEBUG, MSVisualSourceSafeImpl.class, "STDERR - " + err.toString());
				throw new IOException(err.toString());
			}

			proc.waitFor();
		} catch (Exception ex) {
			if(ex.getMessage().indexOf("Undo check out and lose changes?(Y/N)Y") == -1){
				//Log.log(Log.DEBUG, MSVisualSourceSafeImpl.class, ex);
				throw new IOException(ex.getMessage());
			}
		}

		return retVal;
	}
}

