package com.sgbarlow.sourcecontrol;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Enumeration;
import java.util.Properties;

import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPasswordField;
import javax.swing.JScrollPane;
import javax.swing.JTextField;

import org.gjt.sp.jedit.AbstractOptionPane;
import org.gjt.sp.jedit.GUIUtilities;
import org.gjt.sp.jedit.jEdit;
import org.gjt.sp.util.Log;


public class SOSPluginOptionPane 
    extends AbstractOptionPane implements ActionListener
{
    
    final static String PROPERTY = "SOSIntegration.property.";
    final static String NAME = ".name";
    final static String VALUE = ".value";
    
    // Widgets
    private JTextField          m_serverText;
    private JTextField          m_srcsafeiniText;
    private JTextField          m_usernameText;
    private JCheckBox           m_reuseLabel;
    private PropertiesTable     m_workTable;
    private JTextField          m_soscmdText;
    private JLabel              m_projectMap;
    private JPasswordField      m_passwordText;
    
    public SOSPluginOptionPane()
    {
        super("SOS");
    }


    /**
     * Option Pane Layout
     *
    **/
    public void _init()
    {
        GridBagConstraints gridBagConstraints;

        m_serverText        = new JTextField();
        m_soscmdText        = new JTextField();
        m_srcsafeiniText    = new JTextField();
        m_usernameText      = new JTextField();
        m_reuseLabel        = new JCheckBox();
        m_passwordText      = new JPasswordField();
        m_projectMap        = new JLabel();

        /**
         * The domain name or IP address and port of a running
         * SourceOffSite server in the form "servername:port"
         *
         * Example: 10.0.0.1:8880
        **/
        m_serverText.setToolTipText("Example: 10.0.0.1:8880");
        m_serverText.setColumns(10);
        m_serverText.setText(jEdit.getProperty(SourceControlPlugin.OPTION_PREFIX + "sosserver"));
        addComponent("Server Address: ", m_serverText);
        
        /**
         * Path to the SourceOffsite Executable
         *
        **/
        m_soscmdText.setToolTipText("Example: /usr/bin");
        m_soscmdText.setColumns(10);
        m_soscmdText.setText(jEdit.getProperty(SourceControlPlugin.OPTION_PREFIX + "sosfilepath"));
        addComponent("Path to soscmd", m_soscmdText);
        
        /**
         * A valid SourceSafe database that the SOS Server has 
         * access to.  This path is specified under the Databases 
         * tab of the SourceOffSite Server Manager utility
         * Example: C:\Program Files\DevStudio\VSS\YoChat\srcsafe.ini
        **/
        m_srcsafeiniText.setToolTipText("Example: C:\\Program Files\\DevStudio\\VSS\\YoChat\\srcsafe.ini");
        m_srcsafeiniText.setColumns(10);
        m_srcsafeiniText.setText(jEdit.getProperty(SourceControlPlugin.OPTION_PREFIX + "sosdatabase"));
        addComponent("Database file", m_srcsafeiniText);

        /**
         * The SourceSafe user name used for logging into the 
         * SourceOffSite Server.
         *
        **/
        m_usernameText.setToolTipText("Username for sourceOffsite");
        m_usernameText.setColumns(10);
        m_usernameText.setText(jEdit.getProperty(SourceControlPlugin.OPTION_PREFIX + "sosusername"));
        addComponent("Username", m_usernameText);

        /**
         * The SourceSafe password corresponding to the user name 
         * specified by the -name parameter
         *
        **/
        m_passwordText.setToolTipText("Warning: password is stored clear text in property file");
        m_passwordText.setColumns(10);
        m_passwordText.setText(jEdit.getProperty(SourceControlPlugin.OPTION_PREFIX + "sospassword"));
        addComponent("Password", m_passwordText);

        m_reuseLabel.setSelected(true);
        m_reuseLabel.setText("Reuse previous label");
        m_reuseLabel.setToolTipText("If Checked last used label will be the default text on Checkins");
        addComponent(m_reuseLabel);
        
        addSeparator("");
        m_projectMap.setText("Working Directories");
        addComponent(m_projectMap);
        
        m_workTable = new PropertiesTable( SourceControlPlugin.getGlobalProperties(PROPERTY) );
        m_workTable.setToolTipText("Enter Project/Working directory relationships");
        JScrollPane scrollPane = new JScrollPane( m_workTable );
		scrollPane.setPreferredSize( new Dimension( 400, 200 ) );
		addComponent(scrollPane);
    }
    
    
    public void actionPerformed(ActionEvent evt)
    {
        /*
        System.out.println(evt.getActionCommand());
        System.out.println(evt.paramString());
        System.out.println(evt.toString());
        System.out.println(evt.getSource().toString());

        String[] paths = GUIUtilities.showVFSFileDialog(null,
                null, JFileChooser.OPEN_DIALOG, false);

        if (paths != null)
        {
            if (evt.getActionCommand().indexOf("Root") != -1)
            {
                //m_sosProjectRootName.setText(paths[0]);
            }
            else
            {
                m_sosPathName.setText(paths[0]);
            }
        }
        */
    }


    public void _save()
    {
        jEdit.setProperty(SourceControlPlugin.OPTION_PREFIX + "sosfilepath",
                m_soscmdText.getText());

        jEdit.setProperty(SourceControlPlugin.OPTION_PREFIX + "sosdatabase",
                m_srcsafeiniText.getText());
            
        jEdit.setProperty(SourceControlPlugin.OPTION_PREFIX + "sosusername",
                m_usernameText.getText());
                
        jEdit.setProperty(SourceControlPlugin.OPTION_PREFIX + "sospassword",
                new String(m_passwordText.getPassword()));
        
        jEdit.setProperty(SourceControlPlugin.OPTION_PREFIX + "sosserver",
                m_serverText.getText());

        deleteGlobalProperties();
        Properties properties = m_workTable.getProperties();

        try
        {
            // create runtime sos projects
            SourceOffSiteImpl.setSosDir(m_soscmdText.getText());
            SourceOffSiteImpl.setSosDb(m_srcsafeiniText.getText());
            java.util.List projects = SOSProject.factory(properties);
            SourceOffSiteImpl.setSosProjects(projects);

            SourceOffSiteImpl.setUsername(m_usernameText.getText());
            SourceOffSiteImpl.setPassword(new String(m_passwordText.getPassword()));
            SourceOffSiteImpl.setServer(m_serverText.getText());
            
            // now we know that the projects are good
            storeProjectsInJEditProperties(properties);

        }
        catch (Exception ex)
        {
            Log.log(Log.ERROR, SOSPluginOptionPane.class, ex);
            String[] args1 = {ex.getMessage()};
            GUIUtilities.message(null, "sourcecontrol.option.error", args1);
        }
    }


    private void storeProjectsInJEditProperties(Properties props)
    {
        // store projects in jedit properties
        Enumeration propertyNames = props.propertyNames();
        for (int i = 0; propertyNames.hasMoreElements(); i++)
        {
            String name = (String) propertyNames.nextElement();
            String value = props.getProperty(name);

            // Check if there is anything to save.
            if (name == null || value == null)
            {
                continue;
            }

            if (name.trim().length() > 0 && value.trim().length() > 0)
            {
                Log.log(Log.DEBUG, SOSPluginOptionPane.class,
                    "Storing: " + name + " = " + value);
                
                jEdit.setProperty(PROPERTY + (i + 1) + NAME, name);
                jEdit.setProperty(PROPERTY + (i + 1) + VALUE, value);
            }
        }
    }


    private void deleteGlobalProperties()
    {
        String name;
        int counter = 1;
        while ((name = jEdit.getProperty(PROPERTY + counter + NAME)) != null)
        {
            jEdit.setProperty(PROPERTY + counter + NAME, null);
            jEdit.setProperty(PROPERTY + counter + VALUE, null);
            counter++;
        }
    }

}

