package com.sgbarlow.sourcecontrol;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.List;
import java.util.Properties;
import java.util.Vector;

import javax.swing.JOptionPane;

import org.gjt.sp.jedit.Buffer;
import org.gjt.sp.jedit.EditPlugin;
import org.gjt.sp.jedit.GUIUtilities;
import org.gjt.sp.jedit.OptionGroup;
import org.gjt.sp.jedit.View;
import org.gjt.sp.jedit.jEdit;
import org.gjt.sp.jedit.gui.OptionsDialog;
import org.gjt.sp.util.Log;

/**
 *
 */
public class SourceControlPlugin extends EditPlugin {

	 public static final String NAME = "sourcecontrol";
	 public static final String MENU = "sourcecontrol.menu";
	public static final String PROPERTY_PREFIX = "plugin.sourcecontrol.";
	public static final String OPTION_PREFIX = "options.sourcecontrol.";

	 private static List s_projects = null;

	 /**
	 *
	 *
	 */
	public SourceControlPlugin() {}

	 public void start() {
		  init();
	 }

	 private void init() {
		  /*
		  try {
				SourceControlInterface sc = SourceControlInterfaceFactory.getSourceControl();
				sc.init();
		  } catch (Exception ex) {
				Log.log(Log.ERROR, SourceControlPlugin.class, "Error on start: " + ex.getMessage());
		  }
		  */
	 }

	public void stop() {}

	 /**
	  * This should be depreccated and use:
	  * Properties getGlobalProperties(String SourceControlType)
	  *
	 **/
	 public static Properties getGlobalProperties() {
		  Properties properties = new Properties();

		String name;
		int counter = 1;
		while ( ( name = jEdit.getProperty( VSSPluginOptionPane.PROPERTY + counter + VSSPluginOptionPane.NAME ) ) != null ) {
			properties.put( name, jEdit.getProperty( VSSPluginOptionPane.PROPERTY + counter + VSSPluginOptionPane.VALUE ) );
			counter++;
		}
		return properties;
	 }

	 public static Properties getGlobalProperties(String SourceControlType) {
		  Properties properties = new Properties();

		// This is a programming error or no source control system is selected
		if (SourceControlType == null || SourceControlType.length() <= 0) {
				throw new IllegalArgumentException(
					 "Invalid source control system selected");
		  }

		  String name;
		int counter = 1;
		while ( ( name = jEdit.getProperty( SourceControlType + counter + VSSPluginOptionPane.NAME ) ) != null ) {
			properties.put( name, jEdit.getProperty(
					 SourceControlType + counter + VSSPluginOptionPane.VALUE ));

			counter++;
		}
		return properties;
	 }

	 public void createMenuItems(Vector menuItems) {
		  menuItems.addElement(GUIUtilities.loadMenu(MENU));
	 }

	 public void createOptionPanes(OptionsDialog dialog) {
		  //dialog.addOptionPane(new VSSPluginOptionPane());
		  //dialog.addOptionPane(new SourceControlOptionPane());

		  OptionGroup grp = new OptionGroup( NAME );
		  grp.addOptionPane( new SourceControlOptionPane() );
		grp.addOptionPane( new VSSPluginOptionPane() );
		  grp.addOptionPane( new SOSPluginOptionPane() );
		dialog.addOptionGroup( grp );
	 }

	 /**
	 *
	 *
	 */
	 public static void checkoutFile(View view) {
		  String[] args = { "filename" };

		  Log.log(Log.DEBUG, SourceControlPlugin.class, "start checkoutFile");

		  Buffer buff = view.getBuffer();
		  String filePath = buff.getPath();
			 String comment = null;

		  boolean prompt = new Boolean(jEdit.getProperty(
			SourceControlPlugin.OPTION_PREFIX + "promptbeforecheckout")).booleanValue();

			 comment = jEdit.getProperty(SourceControlPlugin.OPTION_PREFIX + "checkoutcomment");
		  if (prompt) {
				  comment = GUIUtilities.input(view,"sourcecontrol.checkout.comment", comment);
				//result = GUIUtilities.confirm(view,"sourcecontrol.checkout.areyousure",
				//	 null, JOptionPane.YES_NO_OPTION,JOptionPane.INFORMATION_MESSAGE);
		  }

		  try {
				if(comment != null) {
					 Log.log(Log.DEBUG, SourceControlPlugin.class, "About to checkout file: " + filePath);
					 SourceControlInterface sc = SourceControlInterfaceFactory.getSourceControl();
					 sc.checkoutFile(filePath, comment);
					 buff.reload(view);
				}
		  } catch (Exception ex) {
				Log.log(Log.ERROR, SourceControlPlugin.class, ex);
				String[] args1 = { ex.getMessage() };
				GUIUtilities.message(view,"sourcecontrol.checkout.error", args1);
		  }
	 }

	 /**
	 *
	 *
	 */
	 public static void checkinFile(View view) {
		  String[] args = { "filename" };
		  Buffer buff = view.getBuffer();
		  String filePath = buff.getPath();

		  String defaultcomment = jEdit.getProperty(SourceControlPlugin.OPTION_PREFIX + "checkoutcomment");
		  String comment = GUIUtilities.input(view,"sourcecontrol.checkin.comment", defaultcomment);

		  try {
				if( comment != null ) {
					 Log.log(Log.DEBUG, SourceControlPlugin.class, "About to checkin file: " + filePath);
					 SourceControlInterface sc = SourceControlInterfaceFactory.getSourceControl();
					 sc.checkinFile(filePath, comment);
					 buff.reload(view);
				}
		  } catch (Exception ex) {
				Log.log(Log.ERROR, SourceControlPlugin.class, ex);
				String[] args1 = { ex.getMessage() };
				GUIUtilities.message(view,"sourcecontrol.checkin.error", args1);
		  }
	 }

	 /**
	 *
	 *
	 */
	 public static void addFile(View view) {
		  String[] args = { "filename" };
		  Buffer buff = view.getBuffer();
		  String file = buff.getPath();
		  String filePath = buff.getVFS().getParentOfPath(buff.getPath());
		  String comment = GUIUtilities.input(view,"sourcecontrol.add.comment", "");

		  try {
				if( comment != null ) {
					 Log.log(Log.DEBUG, SourceControlPlugin.class, "About to add file: " + filePath);
					 SourceControlInterface sc = SourceControlInterfaceFactory.getSourceControl();
					 sc.addFile(file, filePath, comment);
					 buff.reload(view);
				}
		  } catch (Exception ex) {
				Log.log(Log.ERROR, SourceControlPlugin.class, ex);
				String[] args1 = { ex.getMessage() };
				GUIUtilities.message(view,"sourcecontrol.add.error", args1);
		  }
	 }

	 /**
	 *
	 *
	 */
	 public static void undocheckoutFile(View view) {
		  int result = GUIUtilities.confirm(view,"sourcecontrol.undocheckout.areyousure",
		  	null, JOptionPane.YES_NO_OPTION,JOptionPane.INFORMATION_MESSAGE);
		  if (result==0){
		  String[] args = { "filename" };
		  Buffer buff = view.getBuffer();
		  String filePath = buff.getPath();

		  try {
				Log.log(Log.DEBUG, SourceControlPlugin.class, "About to undocheckout file: " + filePath);
				SourceControlInterface sc = SourceControlInterfaceFactory.getSourceControl();
				sc.undoCheckoutFile(filePath);
				buff.reload(view);
		  } catch (Exception ex) {
				Log.log(Log.ERROR, SourceControlPlugin.class, ex);
				String[] args1 = { ex.getMessage() };
				GUIUtilities.message(view,"sourcecontrol.undocheckout.error", args1);
			  }
		  }
	 }

	 /**
	 *
	 *
	 */
	 public static void diffFile(View view) {
		  String[] args = { "filename" };
		  Buffer buff = view.getBuffer();
		  String filePath = buff.getPath();

		  try {
				Log.log(Log.DEBUG, SourceControlPlugin.class, "About to diff file: " + filePath);
				SourceControlInterface sc = SourceControlInterfaceFactory.getSourceControl();
				String diffStuff = sc.diffFile(filePath);

				if ( (diffStuff != null) && (!diffStuff.equals("")) ) {
					 String[] args2 = { diffStuff };

					 File tempFile = File.createTempFile("jEditSourceControlDiff", null);
					 setFileData(tempFile, diffStuff.getBytes());

					 Log.log(Log.DEBUG, SourceControlPlugin.class, "about to open file: " + tempFile.getAbsolutePath());
					 //jEdit.openTemporary(view, tempFile.getParent(), tempFile.getName(), false);
					 jEdit.openFile(view, tempFile.getAbsolutePath());

					 //GUIUtilities.message(view,"sourcecontrol.diff.output", args2);
				}
		}
		catch (Exception ex) {
				Log.log(Log.ERROR, SourceControlPlugin.class, ex);
				String[] args1 = { ex.getMessage() };
				GUIUtilities.message(view,"sourcecontrol.diff.error", args1);
		  }
	 }

	public static String historyFile(View view) {
		Buffer buff = view.getBuffer();
		String filePath = buff.getPath();
		String selectedVersion = "1";


		try {
			Log.log(Log.DEBUG, SourceControlPlugin.class, "About to History file: " + filePath);
			SourceControlInterface sc = SourceControlInterfaceFactory.getSourceControl();

			Vector hs = sc.historyFile(filePath);

			// costruisco l'elenco delle versioni

			if (hs.size() > 0) {
				/*
				JMenu lbl;
				JPopupMenu ver = new JPopupMenu("History of " + buff.getName() );
				for (int i = 0; i < hs.size(); i++) {
					FileVersion fv = (FileVersion)hs.get(i);
					if (fv.labels.size() > 0) {
						lbl = new JMenu(fv.version);
						for (int j = 0; j < fv.labels.size(); j++) {
							lbl.add(((FileLabel)fv.labels.get(j)).label);
						}
						// aggiungo il sottomenu
						URL iconURL = SourceControlPlugin.class.getResource("/icons/bookmark-item.gif");
						ver.add(lbl, new ImageIcon(iconURL));
					} else {
						ver.add(fv.version);
					}
				}
				GUIUtilities.showPopupMenu(ver, view, view.getHeight() / 3, view.getWidth() / 3);
				*/
				FileHistoryGraphPanel f = new FileHistoryGraphPanel(view, buff.getName(), hs, "");
			} else {
				String[] args1 = { "No versions!" };
				GUIUtilities.message(view, "sourcecontrol.history.error", args1);
			}
		} catch (Exception ex) {
			Log.log(Log.ERROR, SourceControlPlugin.class, ex);
			String[] args1 = { ex.getMessage() };
			GUIUtilities.message(view, "sourcecontrol.history.error", args1);
		}

		return selectedVersion;
	}

	public static void getFile(View view, String filePath, String path, String version) {
		try {
			Log.log(Log.DEBUG, SourceControlPlugin.class, "About to getting file: " + filePath);
			SourceControlInterface sc = SourceControlInterfaceFactory.getSourceControl();
			sc.getFile(filePath, path, version);
		} catch (Exception ex) {
			Log.log(Log.ERROR, SourceControlPlugin.class, ex);
			String[] args1 = { ex.getMessage() };
			GUIUtilities.message(view, "sourcecontrol.get.error", args1);
		}
	}

	/**
	*
	*
	*/
	/* 	 public static void jDiffFile(View view) {

	  String[] args = { "filename" };
	  Buffer buff = view.getBuffer();
	  String filePath = buff.getPath();

	  try {
			Log.log(Log.DEBUG, SourceControlPlugin.class, "About to JDiff file: " + filePath);
			SourceControlInterface sc = SourceControlInterfaceFactory.getSourceControl();

			File tempFile = new File(System.getProperty("java.io.tmpdir"), buff.getName());

			if (tempFile.exists())
				tempFile.delete();

			sc.getFile(filePath, tempFile.getParentFile().getPath(), null);

			Log.log(Log.DEBUG, SourceControlPlugin.class, "about to open file: " + tempFile.getAbsolutePath());


			view.unsplit();
			Mode thisMode = buff.getMode();
			//view.splitVertically();
			//Buffer diffBuffer = jEdit.newFile(view);
			//diffBuffer.setMode(thisMode);
			//view.setBuffer(diffBuffer);
			//view.getTextArea().setText(response);

			//jEdit.openTemporary(view, tempFile.getParent(), tempFile.getName(), false);
			Buffer buff2 = jEdit.openFile(view, tempFile.getAbsolutePath());
			buff2.setMode(thisMode);
			jdiff.DualDiff.toggleFor(view);

			tempFile.deleteOnExit();

	  } catch (Exception ex) {
			Log.log(Log.ERROR, SourceControlPlugin.class, ex);
			String[] args1 = { ex.getMessage() };
			GUIUtilities.message(view,"sourcecontrol.diff.error", args1);
	  }
	}
	*/
	public static void setFileData(File file, byte[] data) throws IOException {
	  FileOutputStream out = null;
	  try {
			Log.log(Log.DEBUG, SourceControlPlugin.class, "creating file " + file.getPath());
			out = new FileOutputStream(file);
			Log.log(Log.DEBUG, SourceControlPlugin.class, "number of bytes to write: " + data.length);

			out.write(data);
			out.close();
			Log.log(Log.DEBUG, SourceControlPlugin.class, "written bytes successfully");
	  } catch (IOException ex) {
			if (out != null) {
				 try {
					  out.close();
			} catch (IOException e) {
					  throw new IOException("Error closing outputstream: " + ex.getMessage());
				 }
			}
			throw new IOException("Error closing outputstream: " + ex.getMessage());
	  }
	}
}

