/**
 *@author     Chris Wilson
 *@author     based on Stuart Barlow sgbarlow@yahoo.com MSVisualSourceSafeImpl
 *@created    January 28, 2003
 *
 * TODO:
 *      - Add ability to add a file to project
 *      - Add ability to delete file from project
 *      - Add ability to remove write protection without checkout
 *      - Integrate Diff
 *      - Added better multiple checkout support
 *      - Integrate Source control into File System Browser
 *      - Manage icons for source controled files
 *
 *
**/
package com.sgbarlow.sourcecontrol;

import java.util.*;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.lang.Process;

import org.gjt.sp.util.Log;

// jedit imports
import org.gjt.sp.jedit.*;


public class SourceOffSiteImpl implements SourceControlInterface
{
	 private static String m_sosDir = null;
	 private static String m_sosDb = null;
	 private static List m_projects = null;
	 private static String m_username;
	 private static String m_password;
	 private static String m_server;

	 private static boolean s_isInitialised = false;

	 // Only needed when we send a command to SourceSafe
	 private String m_currentProject = "";
	 private String m_currentWorkDir = "";
	 private java.awt.List m_cmdArray;
	 private SOSProject m_sosProject = null;


	 /**
	  * Default ctor used by the factory.
	  *
	  *@exception  Never Thrown
	  */
	 public SourceOffSiteImpl()
		  throws Exception { }


	 public void init()
		  throws Exception
	 {
		  if (!s_isInitialised)
		  {
				Log.log(Log.DEBUG, SourceOffSiteImpl.class, "start init");
				String sosPath = jEdit.getProperty(SourceControlPlugin.OPTION_PREFIX + "filepath");
				String dbFile = jEdit.getProperty(SourceControlPlugin.OPTION_PREFIX + "sosdatabase");
				List sosProjects = SOSProject.factory(
						  SourceControlPlugin.getGlobalProperties(SOSPluginOptionPane.PROPERTY));

				setSosProjects(sosProjects);
				setSosDir(sosPath);
				setSosDb(dbFile);

				String server = jEdit.getProperty(SourceControlPlugin.OPTION_PREFIX + "sosserver");
				setServer(server);

				String username = jEdit.getProperty(SourceControlPlugin.OPTION_PREFIX + "sosusername");
				setUsername(username);

				String password = jEdit.getProperty(SourceControlPlugin.OPTION_PREFIX + "sospassword");
				setPassword(password);

				Log.log(Log.DEBUG, SourceOffSiteImpl.class, "finish init");

				s_isInitialised = true;
		  }
	 }


	 /**
	  * soscmd -command CheckOutFile
	  *        -server 10.0.0.1:8880 -name "User Name" -password "****"
	  *        -database 'C:\Program Files\DevStudio\VSS\YoChat\srcsafe.ini'
	  *        -project '$/Version1'
	  *        -file build.properties
	  *        -workdir '/home/cwilson/working/project'
	  *
	  *@param  file           Full file name to checkout
	  *@param  comment        not used; kept for compatibility
	  *@exception  Exception  Errors returned by soscmd
	  */
	 public void checkoutFile(String file, String comment)
		  throws Exception
	 {
		  Log.log(Log.DEBUG, SourceOffSiteImpl.class, "checkout file: " + file);
		  File checkoutFile = new File(file);

		  String ssFile = convertFileNameToVSSFileName(file);
		  Log.log(Log.DEBUG, SourceOffSiteImpl.class, "vss file: " + ssFile);

		  m_cmdArray = new java.awt.List();
		  ssCommand();
		  checkoutCommand();
		  serverCommand();
		  getDatabase();
		  userDetails();
		  fileName(checkoutFile);
		  ssFileLocation(checkoutFile);
		  projectDirectory(checkoutFile);

		  execProcess(m_cmdArray.getItems());
	 }


	 public void checkinFile(String file, String comment)
		  throws Exception
	 {
		  File checkinFile = new File(file);
		  String ssFile = convertFileNameToVSSFileName(file);

		  m_cmdArray = new java.awt.List();
		  ssCommand();
		  checkinCommand();
		  serverCommand();
		  getDatabase();
		  userDetails();
		  fileName(checkinFile);
		  ssFileLocation(checkinFile);
		  projectDirectory(checkinFile);
		  addComment(comment);

		  execProcess(m_cmdArray.getItems());
	 }


	 public void undoCheckoutFile(String file)
		  throws Exception
	 {
		  File checkinFile = new File(file);
		  String ssFile = convertFileNameToVSSFileName(file);

		  m_cmdArray = new java.awt.List();
		  ssCommand();
		  undoCheckoutCommand();
		  serverCommand();
		  getDatabase();
		  userDetails();
		  fileName(checkinFile);
		  ssFileLocation(checkinFile);
		  projectDirectory(checkinFile);

		  execProcess(m_cmdArray.getItems());
	 }

	 public void addFile(String file, String filePath, String comment)
		  throws Exception
	 {
		  File addFile = new File(file);
		  String ssFile = convertFileNameToVSSFileName(file);

		  m_cmdArray = new java.awt.List();
		  ssCommand();
		  addCommand();
		  serverCommand();
		  getDatabase();
		  userDetails();
		  fileName(addFile);
		  ssFileLocation(addFile);
		  projectDirectory(addFile);
		  addComment(comment);

		  execProcess(m_cmdArray.getItems());
	 }

	 public String diffFile(String file)
		  throws Exception
	 {
		  throw new Exception("SourceOffSiteImpl - Diff is not supported!");
	 }

	 public void getFile(String file, String path, String version)
		  throws Exception
	 {
		  throw new Exception("SourceOffSiteImpl - get is not supported!");
	 }

	 public Vector historyFile(String file)
		throws Exception
	 {
		  throw new Exception("SourceOffSiteImpl - get is not supported!");
	 }

	 public static void setSosProjects(List projects)
		  throws Exception
	 {
		  m_projects = projects;
	 }

	 public static void setUsername(String name)
	 {
		  m_username = name;
	 }

	 public static void setPassword(String password)
	 {
		  m_password = password;
	 }

	 public static void setServer(String server)
	 {
		  m_server = server;
	 }

	 public static void setSosDir(String directory)
	 {
		  File sourceSafe = new File(directory);
		  if (!sourceSafe.isDirectory())
		  {
				throw new IllegalArgumentException("SourceOffSiteImpl - specified SOS directory does not exist - " + directory);
		  }
		  m_sosDir = directory;
	 }


	 public static void setSosDb(String database)
	 {
		  m_sosDb = database;
	 }


	 //------------------------------------------------------------------------
	 // soscmd commands
	 // -----------------------------------------------------------------------
	 private void checkoutCommand()
	 {
		  m_cmdArray.add("-command");
		  m_cmdArray.add("CheckOutFile");
	 }


	 private void checkinCommand()
	 {
		  m_cmdArray.add("-command");
		  m_cmdArray.add("CheckInFile");
	 }


	 private void undoCheckoutCommand()
	 {
		  m_cmdArray.add("-command");
		  m_cmdArray.add("UndoCheckOut");
	 }

	 private void addCommand()
	 {
		  m_cmdArray.add("-command");
		  m_cmdArray.add("AddFile");
	 }

	 /**
	  * TODO: Add this to the options field
	  */
	 private void serverCommand()
	 {
		  m_cmdArray.add("-server");
		  m_cmdArray.add(m_server);
	 }


	 /**
	  * TODO: Add this to the options field
	  */
	 private void userDetails()
	 {
		  m_cmdArray.add("-name");
		  m_cmdArray.add(m_username);
		  m_cmdArray.add("-password");
		  m_cmdArray.add(m_password);
	 }


	 private void projectDirectory(File file)
	 {
		  String projectWorkFolder = m_sosProject.getWorkingFolder();
		  int startIndex = file.getPath().indexOf(projectWorkFolder)
					  + projectWorkFolder.length();

		  String filename = file.getName();
		  int stopIndex = file.getPath().indexOf(filename);

		  String additional = file.getPath().substring(startIndex, stopIndex);

		  if (additional != null && additional.length() >= 1)
		  {
				additional = "/" + additional;
		  }

		  m_cmdArray.add("-project");
		  m_cmdArray.add(m_sosProject.getProjName() + additional);
	 }


	 private void fileName(File file)
	 {
		  m_cmdArray.add("-file");
		  m_cmdArray.add(file.getName());
	 }


	 private void addComment(String comment)
	 {
		  m_cmdArray.add("-log");
		  m_cmdArray.add(comment);
	 }


	 private void ssFileLocation(File theFile)
	 {
		  m_cmdArray.add("-workdir");
		  m_cmdArray.add(theFile.getParent());
	 }


	 private void getDatabase()
	 {
		  m_cmdArray.add("-database");
		  m_cmdArray.add(m_sosDb);
	 }


	 /**
	  * This has to be called first.
	  */
	 private void ssCommand()
	 {
		  String sep = File.separator;
		  m_cmdArray.add(m_sosDir + sep + "soscmd");
	 }


	 /**
	  *@param  fileName       Description of the Parameter
	  *@return                Description of the Return Value
	  *@exception  Exception  Description of the Exception
	  */
	 private String convertFileNameToVSSFileName(String fileName)
		  throws Exception
	 {
		  StringBuffer strBuf = new StringBuffer(fileName);
		  m_sosProject = null;
		  SOSProject proj = null;
		  Iterator it = m_projects.iterator();
		  while (it.hasNext())
		  {
				proj = (SOSProject) it.next();
				Log.log(Log.DEBUG, SourceOffSiteImpl.class, proj.toString());
				if (fileName.indexOf(proj.getWorkingFolder()) != -1)
				{
					 m_sosProject = proj;
					 break;
				}
		  }

		  if (m_sosProject == null)
		  {
				throw new Exception("SourceOffSiteImpl - file is not in working folder of specified VSS Projects - set projects in the VSSIntegration options");
		  }
		  else
		  {
				m_currentWorkDir = m_sosProject.getWorkingFolder();
				m_currentProject = m_sosProject.getProjName();
				strBuf = strBuf.replace(0, m_sosProject.getWorkingFolder().length(),
						  m_currentWorkDir);
		  }

		  String ssFileName = strBuf.toString();
		  // make sure we are dealing with a VSS name of a file
		  return ssFileName.replace('\\', '/');
	 }


	 private static String execProcess(String[] commands)
		  throws IOException
	 {
		  String retVal = "";
		  if (commands == null)
		  {
				Log.log(Log.DEBUG, SourceOffSiteImpl.class, "execProcess: " + null);
				throw new IOException("Command array was empty");
		  }

		  StringBuffer exeCommand = new StringBuffer();
		  int count = commands.length;

		  for (int i = 0; i < count; i++)
		  {
				exeCommand.append(commands[i] + " ");
		  }

		  Log.log(Log.DEBUG, SourceOffSiteImpl.class, "execProcess: " + exeCommand);
		  Process proc = Runtime.getRuntime().exec(commands);

		  InputStream is = proc.getInputStream();
		  InputStream es = proc.getErrorStream();

		  // read from process stream and write out to log
		  byte[] bufo = new byte[1024];
		  byte[] bufErr = new byte[1024];

		  try
		  {
				int i;
				StringBuffer stdOutBuf = null;
				while ((i = is.read(bufo)) != -1)
				{
					 if (stdOutBuf == null)
					 {
						  stdOutBuf = new StringBuffer(new String(bufo));
					 }
					 else
					 {
						  stdOutBuf.append(new String(bufo));
					 }
				}

				if ((stdOutBuf != null) && (!stdOutBuf.toString().equals("")))
				{
					 String stdOutSt = removeBlankLinesFromEnd(stdOutBuf.toString());
					 retVal = stdOutSt;
					 Log.log(Log.DEBUG, SourceOffSiteImpl.class, "STDOUT - " + stdOutSt);
				}
				else
				{
					 Log.log(Log.DEBUG, SourceOffSiteImpl.class, "STDOUT - " + "nothing");
				}

				StringBuffer stdErrBuf = null;
				while ((i = es.read(bufErr)) != -1)
				{
					 if (stdErrBuf == null)
					 {
						  stdErrBuf = new StringBuffer(new String(bufErr));
					 }
					 else
					 {
						  stdErrBuf.append(new String(bufErr));
					 }
				}

				if ((stdErrBuf != null) && (!stdErrBuf.toString().equals("")))
				{
					 String stdErrSt = removeBlankLines(stdErrBuf.toString());
					 Log.log(Log.DEBUG, SourceOffSiteImpl.class, "STDERR - " + stdErrSt);
					 throw new IOException(stdErrSt);
				}

				proc.waitFor();
		  }
		  catch (Exception ex)
		  {
				//Log.log(Log.DEBUG, SourceOffSiteImpl.class, ex);
				throw new IOException(ex.getMessage());
		  }

		  return retVal;
	 }


	 /**
	  * remove all blank lines from the end of string
	  *
	  *@param  str  Description of the Parameter
	  *@return      Description of the Return Value
	  */
	 public static String removeBlankLinesFromEnd(String str)
	 {
		  String retVal = null;
		  retVal = str.trim();
		  return retVal;
	 }


	 public static String removeBlankLines(String str)
	 {
		  String retVal = null;
		  String lineSep = System.getProperty("line.separator");

		  str.trim();
		  int index = 0;
		  while ((index = str.lastIndexOf(lineSep)) != -1)
		  {
				str = str.substring(0, index);
				str.trim();
		  }

		  retVal = str;
		  return retVal;
	 }
}

