package com.sgbarlow.sourcecontrol;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.Properties;

import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JScrollPane;
import javax.swing.JTextField;

import org.gjt.sp.jedit.AbstractOptionPane;
import org.gjt.sp.jedit.GUIUtilities;
import org.gjt.sp.jedit.jEdit;
import org.gjt.sp.util.Log;


/**
 *
 */
public class VSSPluginOptionPane extends AbstractOptionPane implements ActionListener {

	final static String PROPERTY = "VSSIntegration.property.";
	final static String NAME = ".name";
	final static String VALUE = ".value";

	private JTextField m_vssPathName;
	private JTextField m_sharePathName;
	private JTextField m_usernameText;
	private JPasswordField m_passwordText;

	private PropertiesTable m_table;

	/**
	*/
	public VSSPluginOptionPane() {
		super("VSS");
	}

	public void _init() {
		// vss exe path
		m_vssPathName = new JTextField(jEdit.getProperty(
													 SourceControlPlugin.OPTION_PREFIX + "filepath"));
		JButton pickPath = new JButton(jEdit.getProperty(
													 SourceControlPlugin.OPTION_PREFIX + "choose-path"));
		pickPath.addActionListener(this);
		JPanel pathPanel = new JPanel(new BorderLayout(0, 0));
		pathPanel.add(m_vssPathName, BorderLayout.CENTER);
		pathPanel.add(pickPath, BorderLayout.EAST);

		addComponent(jEdit.getProperty(
							 SourceControlPlugin.OPTION_PREFIX + "directory"),
						 pathPanel);

		m_sharePathName = new JTextField(jEdit.getProperty(
														SourceControlPlugin.OPTION_PREFIX + "sharepath"));

		JPanel sharePanel = new JPanel(new BorderLayout(0, 0));
		sharePanel.add(m_sharePathName, BorderLayout.CENTER);
		addComponent(jEdit.getProperty(
							 SourceControlPlugin.OPTION_PREFIX + "vssshare"),
						 sharePanel);

		m_usernameText = new JTextField(jEdit.getProperty(
													  SourceControlPlugin.OPTION_PREFIX + "vssusername"));

		JPanel userPanel = new JPanel(new BorderLayout(0, 0));
		userPanel.add(m_usernameText, BorderLayout.CENTER);
		addComponent("Username", userPanel);

		m_passwordText = new JPasswordField(jEdit.getProperty(SourceControlPlugin.OPTION_PREFIX + "vsspassword"));
		m_passwordText.setToolTipText("Warning: password is stored clear text in property file");
		JPanel pwdPanel = new JPanel(new BorderLayout(0, 0));
		pwdPanel.add(m_passwordText, BorderLayout.CENTER);
		addComponent("Password", pwdPanel);

		// projects
		m_table = new PropertiesTable( SourceControlPlugin.getGlobalProperties() );
		JScrollPane scrollPane = new JScrollPane( m_table );
		scrollPane.setPreferredSize( new Dimension( 300, 200 ) );
		addComponent( jEdit.getProperty(
							  SourceControlPlugin.OPTION_PREFIX + "vssprojects"),
						  scrollPane );
	}

	public void actionPerformed(ActionEvent evt) {
		System.out.println( evt.getActionCommand() );
		System.out.println( evt.paramString() );
		System.out.println( evt.toString() );
		System.out.println( evt.getSource().toString() );

		String[] paths = GUIUtilities.showVFSFileDialog(null,
							  null, JFileChooser.OPEN_DIALOG, false);

		if (paths != null) {
			if ( evt.getActionCommand().indexOf("Root") != -1 ) {
				//m_vssProjectRootName.setText(paths[0]);
			}
			else {
				m_vssPathName.setText(paths[0]);
			}
		}
	}

	public void _save() {
		jEdit.setProperty(SourceControlPlugin.OPTION_PREFIX + "filepath",
								m_vssPathName.getText());

		jEdit.setProperty(SourceControlPlugin.OPTION_PREFIX + "sharepath",
								m_sharePathName.getText());

		jEdit.setProperty(SourceControlPlugin.OPTION_PREFIX + "vssusername",
								m_usernameText.getText());
		jEdit.setProperty(SourceControlPlugin.OPTION_PREFIX + "vsspassword",
								new String(m_passwordText.getPassword()));

		// get rid of old settings
		deleteGlobalProperties();
		// put in the new ones

		Properties properties = m_table.getProperties();

		try {
			// create runtime vss projects
			MSVisualSourceSafeImpl.setSSDir(m_vssPathName.getText());
			java.util.List projects = VSSProject.factory(properties);
			MSVisualSourceSafeImpl.setUsrAndPwd(m_usernameText.getText(), new String(m_passwordText.getPassword()));
			MSVisualSourceSafeImpl.setShareDir(m_sharePathName.getText());

			MSVisualSourceSafeImpl.setVSSProjects(projects);
			// now we know that the projects are good
			storeProjectsInJEditProperties(properties);

		} catch (Exception ex) {
			Log.log(Log.ERROR, VSSPluginOptionPane.class, ex);
			String[] args1 = { ex.getMessage() };
			GUIUtilities.message(null, "sourcecontrol.option.error", args1);
		}
	}

	private void storeProjectsInJEditProperties(Properties props) {
		Iterator it = props.values().iterator();
		while (it.hasNext()) {
			String avalue = (String)it.next();
			Log.log(Log.DEBUG, VSSPluginOptionPane.class, "project value: " + avalue);
		}

		// store projects in jedit properties
		Enumeration propertyNames = props.propertyNames();
		for ( int i = 0; propertyNames.hasMoreElements(); i++ ) {
			String name = (String) propertyNames.nextElement();
			String value = props.getProperty( name );

			// Check if there is anything to save.
			if ( name == null || value == null )
				continue;

			if ( name.trim().length() > 0 && value.trim().length() > 0 ) {
				jEdit.setProperty( PROPERTY + ( i + 1 ) + NAME, name );
				jEdit.setProperty( PROPERTY + ( i + 1 ) + VALUE, value );
			}
		}
	}

	private void deleteGlobalProperties() {
		String name;
		int counter = 1;
		while ( ( name = jEdit.getProperty( PROPERTY + counter + NAME ) ) != null ) {
			jEdit.setProperty( PROPERTY + counter + NAME, null );
			jEdit.setProperty( PROPERTY + counter + VALUE, null );
			counter++;
		}
	}
}

